;====================================================
;
; dtv 1541 loader with DMA
;
; by peiselulli in 2008
;
;====================================================



;----------------------------------------------------
;
; file system interface variables
;
;----------------------------------------------------

load_addr_low = $56
load_addr_mid = $57
load_addr_high = $58
load_addr_ignore = $59

bytes_sector_left = load_addr_low
sector_offset = load_addr_mid

;----------------------------------------------------
;
; zeropage interface variables
;
;----------------------------------------------------

disk_start_track = $49
disk_end_track = $4a

dest_addr_low_byte = $4b
dest_addr_mid_byte = $4c
dest_addr_high_byte = $4d

;----------------------------------------------------
;
; file system local variables
;
;----------------------------------------------------


sector_addr = $51
sector_addr_high = $52
current_track = $53
file_name_low = $54
file_name_high = $55
file_first_block = $5a

store_y = load_addr_high
store_x = file_first_block

;----------------------------------------------------
;
; zeropage variables
;
;----------------------------------------------------

help1 = $42
help2 = $43
help3 = $44
help4 = $45
help5 = $46
sectors_to_go_low = $47
sectors_to_go_high = $48
temp_addr_low = $4e
temp_addr_mid = $4f
temp_addr_high = $50

;----------------------------------------------------
;----------------------------------------------------
;
; FILE SYSTEM
;
;----------------------------------------------------
;----------------------------------------------------

;----------------------------------------------------
; file getbyte
; in a byte
; carry set if no character left
;----------------------------------------------------

file_getbyte:
    sty store_y
    lda bytes_sector_left
    bne get_from_current_sector
    stx store_x
    jsr fastload_initalize_register
    jsr read_next_sector
    bne no_byte_left
    jsr step_to_next_track
    ldx store_x
    ldy #$00
    lda (sector_addr),y
    bne not_last_sector_getb
    iny
    lda (sector_addr),y
    !byte $2c    
not_last_sector_getb:
    lda #254
    sta bytes_sector_left 
    ldy #$02
    !byte $2c
            
get_from_current_sector
    ldy sector_offset
    lda (sector_addr),y
    iny
    sty sector_offset
    dec bytes_sector_left
    ldy store_y
    clc
    rts
    
no_byte_left:
    ldx store_x
    ldy store_y
    sec
    rts
    
;----------------------------------------------------
; file_open
; in a/x name (a low)
; if load_addr_ignore is non-zero, read also start addr
; carry set -> failed
;----------------------------------------------------

file_open:
    pha
    txa
    pha
    jsr fastload_initalize_register
    pla
    tax
    pla
    jsr search_file_start
    cmp #$00
    beq open_failed
    jsr read_sector ; read the sector with that content
    ldy #$00
    lda (sector_addr),y
    bne not_last_sector
    iny
    lda (sector_addr),y
    tax
    !byte $2c    
not_last_sector:
    ldx #254
    ldy #$02
    lda load_addr_ignore
    bne ignore_load_addr
    dex
    dex
    iny
    iny
ignore_load_addr:
    stx bytes_sector_left
    sty sector_offset
    clc
    rts

open_failed
file_not_found:
    lda #-1
    sec
    rts

;----------------------------------------------------
; file_load
; in a/x name (a low)
; in load_addr_low, load_addr_mid, load_addr_high
;     start addr
;  if all are zero, take the original one
; if load_addr_ignore is non-zero, read also start addr
;----------------------------------------------------

file_load:
    pha
    txa
    pha
    jsr fastload_initalize_register
    pla
    tax
    pla
    jsr search_file_start
    cmp #$00
    beq file_not_found
    jsr read_sector        ; read the sector with that content
    lda load_addr_ignore
    beq +
    lda #$00
    sta file_first_block
    +bra file_load_no_new_addr
+   lda #$01
    sta file_first_block
    lda load_addr_low
    ora load_addr_mid
    ora load_addr_high
    bne file_load_no_new_addr
    ldy #$02
    lda (sector_addr),y
    sta load_addr_low
    iny
    lda (sector_addr),y
    sta load_addr_mid
    +bra file_load_no_new_addr
file_load_next_sector:
    lda #$00
    sta file_first_block
file_load_no_new_addr:

    ldy #$00
    lda (sector_addr),y
    beq last_sector
    ;copy data
    
    +waitblit
    
    lda load_addr_low
    sta blt_d_low
    lda load_addr_mid
    sta blt_d_mid
    lda load_addr_high
    sta blt_d_high

    lda #$02
    ldy file_first_block
    beq +
    lda #$04
+   sta blt_a_low
    lda sector_addr_high
    sta blt_a_mid
    ldx #$00
    stx blt_a_high
    stx blt_length_mid
    lda #254
    ldy file_first_block
    beq +
    lda #254-2
+   sta blt_length_low
    
    ldx #$0b
    stx blt_ctrl_0

    clc
    adc load_addr_low
    sta load_addr_low
    bcc +
    inc load_addr_mid
    bne +
    inc load_addr_high
+         
    jsr read_next_sector
    php
    jsr step_to_next_track
    plp
    beq file_load_next_sector
    
    
last_sector:
    ;copy data
    +waitblit
    lda load_addr_low
    sta blt_d_low
    lda load_addr_mid
    sta blt_d_mid
    lda load_addr_high
    sta blt_d_high

    lda #$02
    ldy file_first_block
    beq +
    lda #$04
+   sta blt_a_low
    lda sector_addr_high
    sta blt_a_mid
    ldx #$00
    stx blt_a_high
    stx blt_length_mid
    ldy #$01
    lda (sector_addr),y
    ldy file_first_block
    beq +
    sec
    sbc #$02
+   sta blt_length_low
    
    ldx #$0b
    stx blt_ctrl_0

    clc
    adc load_addr_low
    sta load_addr_low
    bcc +
    inc load_addr_mid
    bne +
    inc load_addr_high
+         
    jmp deinit_dma_and_blitter
    
;----------------------------------------------------
;
; search_file_start
; in a/x name (a low)
; in a/x track/sector will be returned, 
;             0/0 = file not found
;
;----------------------------------------------------

search_file_start:
    sta file_name_low
    stx file_name_high
    ; read first 18,0
    lda #18
    ldx #$00
    jsr read_sector_cached
search_file_start_loop:
    jsr read_next_sector_cached
    bne search_file_start_fail
    lda #$05
    sta help5
search_file_start_inner_loop:
    sta help3
    lda sector_addr_high
    sta help4
    lda file_name_low
    sta help1
    lda file_name_high
    sta help2
    jsr regexp
    beq search_file_start_found
    lda help5
    clc
    adc #$20
    sta help5
    bcc search_file_start_inner_loop
    bcs search_file_start_loop

search_file_start_found:
    ldy help5
    dey
    lda (sector_addr),y
    tax
    dey
    lda (sector_addr),y
    rts    

search_file_start_fail:
    lda #$00
    tax
    rts

;----------------------------------------------------
;
; step_to_next_track
; in sector_addr / sector_addr_high old block
; in sector_addr / sector_addr_high will be returned
; 
;
;----------------------------------------------------

step_to_next_track:
    ldy #$00
    lda (sector_addr),y
    beq step_to_next_track_end
    cmp current_track
    beq step_to_next_track_end
    jsr step_to
    
step_to_next_track_end:
    rts
            
;----------------------------------------------------
;
; read_next_sector
; in sector_addr / sector_addr_high old block
; in sector_addr / sector_addr_high will be returned
; a = 0 + ZERO set -> success
;
;----------------------------------------------------

read_next_sector
    ldy #$00
    lda (sector_addr),y
    beq regexp_fail
    iny
    lda (sector_addr),y
    tax
    dey
    lda (sector_addr),y
    jsr read_sector
    lda #$00
    rts
    
;----------------------------------------------------
;
; read_next_sector_cached
; in sector_addr / sector_addr_high old block
; in sector_addr / sector_addr_high will be returned
; a = 0 + ZERO set -> success
;
; chaced version for directories
;----------------------------------------------------

read_next_sector_cached
    ldy #$00
    lda (sector_addr),y
    beq regexp_fail
    iny
    lda (sector_addr),y
    tax
    dey
    lda (sector_addr),y
    jsr read_sector_cached
    lda #$00
    rts


;----------------------------------------------------
;
; check_disk_name
; in a/x pointer (a low)
; a = 0, zero set -> name ok
; a != 0, zero clear -> name not ok
;
;----------------------------------------------------

check_disk_name:
    pha
    txa
    pha
    jsr fastload_initalize_register
    lda #18
    ldx #$00
    jsr read_sector_cached
    pla
    sta help2
    pla
    sta help1
    lda #$90
    sta help3
    lda sector_addr_high
    adc #$00
    sta help4
    ; now regexp follows, no jump needed !
     
;----------------------------------------------------
;
; regexp
; in help1/help2 ptr 1 with wildcards
; in help3/help4 ptr 2
;
;----------------------------------------------------

regexp:
    ldy #$00
regexploop:
    lda (help1),y
    beq regexp_ok
    cmp #"?"
    beq regexp_question_mark
    cmp #"*"
    beq regexp_ok
    cmp (help3),y
    bne regexp_fail
regexp_question_mark:
    iny
    +bra regexploop

regexp_ok:
    lda #0
    rts
    
regexp_fail:
    lda #-1
    rts        


;----------------------------------------------------
;
; read a sector and give back the address in
; sector_addr / sector_addr_high
;              in a track
;              in x sector
;
; cached version for directories
;----------------------------------------------------

read_sector_cached:
    ;+bra read_sector
    ;sector caching only for track 18
    cmp #18
    bne read_sector
    ;search for sector in list
    ldy #sector_cache_numbers-1
-   txa
    cmp sector_cache_tab,y
    beq read_sector_cached_found
    lda sector_cache_tab,y
    bmi read_sector_cached_found_free_place    
    dey
    bpl -
    ;no free sector in list, no cached block found
    lda #18
    +bra read_sector

read_sector_cached_found:
    tya
    clc
    adc #>sectorcache
    sta sector_addr_high
    lda #$00
    sta sector_addr
    rts
    
read_sector_cached_found_free_place:
    ;found free place, read
    txa
    sta sector_cache_tab,y
    tya
    pha
    lda #18
    jsr read_sector
    
    ;copy sector, but return the old buffer
    ;(blitter transfer not in zero cycles !)
    +waitblit
    pla
    clc
    adc #>sectorcache
    sta blt_d_mid
    lda #^sectorcache
    sta blt_d_high
    lda #<sectorcache
    sta blt_d_low
    ldy #$00
    sty blt_a_high
    sty blt_length_low
    iny
    sty blt_length_mid   
        
    lda sector_addr
    sta blt_a_low
    lda sector_addr_high
    sta blt_a_mid
    lda #$0b
    sta blt_ctrl_0
    +waitblit
    lda #$00
    rts
    
    

;----------------------------------------------------
;
; read a sector and give back the address in
; sector_addr / sector_addr_high
;              in a track
;              in x sector
;
;----------------------------------------------------


read_sector:
    cmp current_track
    beq read_sector_no_read
    sta disk_start_track
    sta disk_end_track
    sta current_track
    txa
    pha
    lda #<trackbuffer
    sta dest_addr_low_byte
    lda #>trackbuffer
    sta dest_addr_mid_byte
    lda #^trackbuffer
    sta dest_addr_high_byte
    jsr disk_transfer
    pla
    tax
read_sector_no_read:
    lda #$00
    sta sector_addr
    txa
    clc
    adc #>trackbuffer
    sta sector_addr_high
    rts    


;----------------------------------------------------
;----------------------------------------------------
;
; LOW LEVEL FUNCTION
;
;----------------------------------------------------
;----------------------------------------------------

;----------------------------------------------------
;
; definitions for the commands of the floppy
;
;----------------------------------------------------
        
; bit 7&6 == 0 => starttrack to read, endtrack follows in next byte
; bit 7 == 0, bit 6 == 1 => step to

COMMAND_RESET = $80
COMMAND_MOTOR_OFF = $81
COMMAND_LED_ON = $82
COMMAND_LED_OFF = $83
COMMAND_DISKCHANGE = $84


;----------------------------------------------------
;
; helper for speed debugging
;
;----------------------------------------------------

cyclecount = 0
printascii = $ffd2


;----------------------------------------------------
;
; memory addresses in DTV memory
;
;----------------------------------------------------

blockbuffer = $8000
dmabuffer = $8200
dmalength = $1c00
trackbuffer = dmabuffer + dmalength
sectorcache = trackbuffer+21*$100
sectorcache_end = $c000
sector_cache_numbers = (sectorcache_end-sectorcache)/$100

sector_cache_tab = dmabuffer-sector_cache_numbers
  
dma_repitions = 3 ; slowdown factor for DMA disk transfer

;----------------------------------------------------
;
; memory addresses in floppy memory
;
;----------------------------------------------------

floppyblock = $0140
floppyblockend = floppyblock + $145+$0a
floppyblockstransferred = floppyblockend+4
floppystart = $0300

;----------------------------------------------------
;
; copy code to floppy and execute it
;
;----------------------------------------------------        

fastload_initialize:     
        ;disable turbo
        +sac $99
        lda #$00
        +sac $00
        lda #$36
        sta $01
        jsr initfloppy
        jsr copyfloppycode
        jsr floppyexec
        
        ; +enable_extended
         ;ENABLE TURBO
         +sac $99
         LDA #$03
         +sac $00
        
        
        lda #$3f
        sta $dd02
        lda $dd00
        and #$df
        sta $dd00
        ; for file system
        ldx #sector_cache_numbers-1
        lda #-1
        sta current_track 
-       sta sector_cache_tab,x
        dex
        bpl -
        
fastload_initalize_register:        
        ;initialize dma controller
        +waitdma
        ldx #$1f
        lda #$00
-       sta $d300,x
        dex
        bpl -
        sta dma_src_low
        sta dma_ctrl
        lda #$dd
        sta dma_src_mid
        lda #$80
        sta dma_src_high
        lda #<dmabuffer
        sta dma_dst_low
        lda #>dmabuffer
        sta dma_dst_mid
        lda #$40+(^dmabuffer)
        sta dma_dst_high
        lda #$00
        sta dma_src_step_low
        sta dma_src_step_mid
        sta dma_dst_step_mid
        sta dma_dst_step_low
        sta dma_dst_llength_mid
        sta dma_dst_mod_mid
        lda #$01
        sta dma_dst_mod_low
        lda #dma_repitions-1
        sta dma_dst_llength_low
        lda #$00
        sta dma_length_low
        lda #>(dmalength * dma_repitions)
        sta dma_length_mid
        lda #$02
        sta dma_mod_enable
        lda #$00
        sta dma_continue
        ;initalize blitter
        +waitblit
        ldx #$1f
        lda #$00
-       sta $d320,x
        dex
        bpl -
        lda #$10
        sta blt_a_frac_step
        sta blt_d_frac_step
        ldx #$00
        stx blt_a_llength_low
        stx blt_d_llength_low
        inx
        stx blt_a_llength_mid
        stx blt_d_llength_mid
        stx blt_ctrl_1 ; disable channel b, no transparent
        lda #%00011000
        sta blt_shift

        rts

;----------------------------------------------------
;
; check for correct checksums in the block
;
;----------------------------------------------------        
        
check:
        lda #$08        ; header type
        cmp blockbuffer
        bne check_error
        lda #$0f        ; two bytes end of header
        cmp blockbuffer+6
        bne check_error
        cmp blockbuffer+7
        bne check_error
        lda #$07        ; block type
        cmp blockbuffer+8
        bne check_error
        lda blockbuffer+$10a ; block end
        bne check_error
        ;checksum over header
        ldx #$02
        lda blockbuffer+1
-       eor blockbuffer,x
        inx
        cpx #$06
        bne -
        cmp #$00
        bne check_error
        ;checksum over data
        ldx #$00
        lda blockbuffer+$109
-       eor blockbuffer+9,x
        inx
        bne -
        cmp #$00
        bne check_error
        rts
        
check_error:
        inc $d020
        +bra check_error

;----------------------------------------------------
;
; sends an "initalize" to to floppy
;
;----------------------------------------------------
 
initfloppy:
        lda #$08
        sta $ba
        jsr $ffb4
        lda #$6f
        sta $b9
        jsr $ff96
-       jsr $ffa5
        cmp #$0d
        bne -
        jsr $ffab
        
        lda #$08
        jsr $ffb1
        lda #$6f
        jsr $ff93
        lda #$49
        jsr $ffa8
        jmp $ffae
        rts
    
;----------------------------------------------------
;
; copies the code to the floppy memory
;
;----------------------------------------------------

copyfloppycode:
        lda #<floppycode
        sta help1
        lda #>floppycode
        sta help2
        lda #<floppystart
        sta help3
        lda #>floppystart
        sta help4
        lda $01
        sta old_01+1
copyfloppystreaml:        
        lda #$08
        jsr $ffb1
        lda #$6f
        jsr $ff93
        lda #$4d
        jsr $ffa8
        lda #$2d
        jsr $ffa8
        lda #$57
        jsr $ffa8
        lda help3
        jsr $ffa8
        lda help4
        jsr $ffa8
        lda #$20
        jsr $ffa8
        ldy #$00
copyfloppybyte:
        sei
        ldx #$30
        stx $01
        lda (help1),y
old_01: ldx #$36
        stx $01
        cli
        jsr $ffa8
        iny
        cpy #$20
        bne copyfloppybyte
        jsr $ffae
        lda help1
        clc
        adc #$20
        sta help1
        bcc +
        inc help2
+       lda help3
        clc
        adc #$20
        sta help3
        bcc +
        inc help4
+       lda help1
        cmp #<floppycodeend
        lda help2
        sbc #>floppycodeend
        bcc copyfloppystreaml
        rts

;----------------------------------------------------
;
; execute the code in the floppy at addr "floppystart"
;
;----------------------------------------------------


floppyexec:
        ; start the program
        lda #$08
        jsr $ffb1
        lda #$6f
        jsr $ff93
        lda #$4d
        jsr $ffa8
        lda #$2d
        jsr $ffa8
        lda #$45
        jsr $ffa8
        lda #<floppystart
        jsr $ffa8
        lda #>floppystart
        jsr $ffa8
        jsr $ffae
        rts


;----------------------------------------------------
;
; copy the block dependent from its sector and track
;
;----------------------------------------------------
 
copy_disk_block:
        lda dest_addr_low_byte
        sta temp_addr_low
        lda dest_addr_mid_byte
        sta temp_addr_mid
        lda dest_addr_high_byte
        sta temp_addr_high
        ldx blockbuffer+3
-       cpx disk_start_track
        beq tracks_are_calcculated
        dex
        lda tracks_per_sec_tab,x
        clc
        adc temp_addr_mid
        sta temp_addr_mid
        bcc +
        inc temp_addr_high
+       +bra -

tracks_are_calcculated:
        lda blockbuffer+2
        clc
        adc temp_addr_mid
        sta temp_addr_mid
        bcc no_sector_overflow
        inc temp_addr_high
no_sector_overflow:
        +waitblit

        lda #<(blockbuffer+9)
        sta blt_a_low
        lda #>(blockbuffer+9)
        sta blt_a_mid
        lda #^(blockbuffer+9)
        sta blt_a_high
        ldx #$00
        stx blt_length_low
        inx
        stx blt_length_mid
        
        lda temp_addr_low
        sta blt_d_low
        lda temp_addr_mid
        sta blt_d_mid
        lda temp_addr_high
        sta blt_d_high
        lda #$0b
        sta blt_ctrl_0
        rts

        
;----------------------------------------------------
;
; sends byte in accu to floppy
;
;----------------------------------------------------


send_byte_to_floppy:
        sta help1
-       lda $dd00
        and #$40
        bne -
        ldx #$08
send_bit_to_floppy:
        asl help1
        bcc send_low_bit
        lda $dd00
        ora #$28
        sta $dd00
        +bra send_high_bit
send_low_bit
        lda $dd00
        and #$df
        ora #$08
        sta $dd00
send_high_bit
        ldy #$08
-       dey
        bne -        
        lda $dd00
        and #$f7
        sta $dd00
        ldy #$08
-       dey
        bne -        
        dex
        bne send_bit_to_floppy
        lda $dd00
        and #$df
        sta $dd00
-       lda $dd00
        and #$40
        beq -
        
        ldy #$04
-       dey
        bne -        
        lda $dd00
        ora #$08
        sta $dd00
        ldy #$04
-       dey
        bne -        
        lda $dd00
        and #$f7
        sta $dd00        
        rts

;----------------------------------------------------
;
; disk_change
;
;----------------------------------------------------

disk_change:
        ; for file system
        ldx #sector_cache_numbers-1
        lda #-1
        sta current_track 
-       sta sector_cache_tab,x
        dex
        bpl -
        lda #COMMAND_DISKCHANGE
        jsr send_byte_to_floppy
        lda #COMMAND_DISKCHANGE
        jmp send_byte_to_floppy

;----------------------------------------------------
;
; step_to
;
;----------------------------------------------------

step_to:
        ora #$40
        !byte $2c
        
;----------------------------------------------------
;
; led_off
;
;----------------------------------------------------

led_off:
        lda #COMMAND_LED_OFF
        !byte $2c
        
;----------------------------------------------------
;
; led_on
;
;----------------------------------------------------

led_on:
        lda #COMMAND_LED_ON
        !byte $2c

;----------------------------------------------------
;
; motor_off
;
;----------------------------------------------------

motor_off:
        lda #COMMAND_MOTOR_OFF
        !byte $2c

;----------------------------------------------------
;
; floppy_reset
;
;----------------------------------------------------

floppy_reset:
        lda #COMMAND_RESET
        jmp send_byte_to_floppy


;----------------------------------------------------
;
; read_tracks
;
;----------------------------------------------------

read_tracks:
        jsr fastload_initalize_register
        jsr disk_transfer
deinit_dma_and_blitter
        +waitblit
        ldx #$3f
        lda #$00
-       sta $d320,x
        dex
        bpl -
        rts

;----------------------------------------------------
;
; macro for deccoding one bit of the DMA stream
; .v == 0 -> take one bit with zero as clock bit
; .v != 0 -> take one bit with one as clock bit
;
;----------------------------------------------------

!MACRO decode_bit .v {
        +align4
        iny
        bne +
        inc help4
        nop
+
        lda (help3),y
        asl
        asl
        rol help2
        +bra .inc_lowbyte
        nop
.inc_highbyte:
        inc help4       
        +bra .doload
.inc_lowbyte         
        iny
        beq .inc_highbyte
.doload
        lda (help3),y
        !if .v = 0 {
        bpl .inc_lowbyte
        }
        !if .v <> 0 {
        bmi .inc_lowbyte
        }
}

                 
;----------------------------------------------------
;
; disk_transfer
;
;----------------------------------------------------

disk_transfer:
        ; DMA must be initialized !!!!
        lda #0
        sta sectors_to_go_low
        sta sectors_to_go_high
        ldx disk_start_track
        !byte $24
-       inx
        lda tracks_per_sec_tab,x
        clc
        adc sectors_to_go_low
        sta sectors_to_go_low
        bcc +
        inc sectors_to_go_high
+       cpx disk_end_track
        bne -
        
        lda disk_start_track
        jsr send_byte_to_floppy
        lda disk_end_track
        jsr send_byte_to_floppy

sectorloop:
        ; wait for dma and blitter finished !
        +waitdma
        +waitblit
        
waitforfloppy:
        lda $dd00
        bmi waitforfloppy
        
        lda $dd00
        ora #$08
        sta $dd00

        ldx #$4
-       dex
        bne -
        php
        sei
        ldx #$0d
        stx dma_ctrl
        and #$f7
        sta $dd00
        plp
        +cyclestart
        ; decode block
        lda #<dmabuffer
        sta help3
        lda #>dmabuffer
        sta help4
        ldy #$00
        ;first, find sync
-       iny
        lda (help3),y
        and #$c0
        cmp #$c0
        bne -        
        
-       iny        ; overun high byte impossible
        lda (help3),y
        asl
        bmi -
        ;now comes the data
        ldx #$00
getbyteloop:
        +decode_bit 1
        +decode_bit 0
        +decode_bit 1
        +decode_bit 0
        +decode_bit 1
        
        lda help2
        and #$1f
        stx help1
        tax
        lda gcrrevtab_shifted,x
        ldx help1
        sta help5

        +decode_bit 0
        +decode_bit 1
        +decode_bit 0
        +decode_bit 1
        +decode_bit 0
        
        lda help2
        and #$1f
        stx help1
        tax
        lda gcrrevtab,x
        ldx help1
        ora help5
        sta blockbuffer,x

        inx
        beq getbyteloop2
        jmp getbyteloop

getbyteloop2:
        +decode_bit 1
        +decode_bit 0
        +decode_bit 1
        +decode_bit 0
        +decode_bit 1
        
        lda help2
        and #$1f
        stx help1
        tax
        lda gcrrevtab_shifted,x
        ldx help1
        sta blockbuffer+$100,x

        +decode_bit 0
        +decode_bit 1
        +decode_bit 0
        +decode_bit 1
        +decode_bit 0
        
        lda help2
        and #$1f
        stx help1
        tax
        lda gcrrevtab,x
        ldx help1
        ora blockbuffer+$100,x
        sta blockbuffer+$100,x

        inx
        cpx #$0c
        beq +
        jmp getbyteloop2
+
        ; dma transfer is shurely ended !        
        +cyclestop
        !if cyclecount <> 0 {
        lda #$13
        jsr $ffd2
        }
        +cycleprint
        jsr copy_disk_block
        jsr check
        lda sectors_to_go_low
        bne +
        dec sectors_to_go_high
+       dec sectors_to_go_low

        lda sectors_to_go_low
        ora sectors_to_go_high
        beq +
        jmp sectorloop
+       rts


;----------------------------------------------------
;
; table for holding number of blocks
;
;----------------------------------------------------


tracks_per_sec_tab:
        !byte -1
        !for i,17 {
        !byte 21
        }
        !for i,7 {
        !byte 19
        }
        !for i,6 {
        !byte 18
        }
        !for i,10 {
        !byte 17
        }


;----------------------------------------------------
;
; tables for decoding blocks
;
;----------------------------------------------------
        
gcrrevtab:
        !byte -1
        !byte -1
        !byte -1
        !byte -1
        !byte -1
        !byte -1
        !byte -1
        !byte -1
        !byte -1
        !byte 8
        !byte 0
        !byte 1
        !byte -1
        !byte $c
        !byte 4
        !byte 5
        !byte -1
        !byte -1
        !byte 2
        !byte 3
        !byte -1
        !byte $f
        !byte 6
        !byte 7
        !byte -1
        !byte 9
        !byte $a
        !byte $b
        !byte -1
        !byte $d
        !byte $e
        !byte -1

gcrrevtab_shifted:
        !byte -1
        !byte -1
        !byte -1
        !byte -1
        !byte -1
        !byte -1
        !byte -1
        !byte -1
        !byte -1
        !byte 8*$10
        !byte 0*$10
        !byte 1*$10
        !byte -1
        !byte $c*$10
        !byte 4*$10
        !byte 5*$10
        !byte -1
        !byte -1
        !byte 2*$10
        !byte 3*$10
        !byte -1
        !byte $f*$10
        !byte 6*$10
        !byte 7*$10
        !byte -1
        !byte 9*$10
        !byte $a*$10
        !byte $b*$10
        !byte -1
        !byte $d*$10
        !byte $e*$10
        !byte -1


;----------------------------------------------------
;----------------------------------------------------
;
; CODE FOR FLOPPY
; MAIN
;
;----------------------------------------------------
;----------------------------------------------------

DISK_TEMP = $1b
CURRENT_TRACK = $2c
SECTORS_TO_GO = $2d
TRACK_POS_WANTED = $2d ; never used synchron with "SECTORS_TO_GO"
DTV_BYTE = $2d ; never used synchron with "SECTORS_TO_GO" and "TRACK_POS_WANTED"
OLD_CHANGE_STAT = $37
CHANGE_COUNTER =$3b


floppycode:
                
!pseudopc floppystart {


    sei
    ldx #$3f
    txs
    lda #18
    sta CURRENT_TRACK
    lda #$00
    sta $1800
command_loop:
    jsr getbyte_from_dtv
    bmi extra_command
    cmp #$40
    bcc load_tracks
    and #$3f
    jsr head_step_to
    jmp command_loop
extra_command:
    cmp #COMMAND_RESET
    bne noreset 
reset:
    lda #17
    jsr head_step_to
    sei
    ldx #$00
    txa
-   sta $00,x
    inx
    bne -
    jmp ($fffc)
noreset:
    cmp #COMMAND_MOTOR_OFF
    bne nomotor_off
    lda $1c00
    and #$fb
    sta $1c00
    jmp do_led_off
nomotor_off:
    cmp #COMMAND_LED_ON
    bne no_led_on
    lda $1c00
    ora #$08
    sta $1c00
    bne command_loop ; unconditional
no_led_on:
    cmp #COMMAND_LED_OFF
    bne no_led_of    
do_led_off:
    lda $1c00
    and #$f7
    sta $1c00
    jmp command_loop
no_led_of:
    cmp #COMMAND_DISKCHANGE
    bne no_known_command
    jsr diskchange
    jmp command_loop
no_known_command:
    jmp error    
        

    
load_tracks:
    sta starttrack+1
    jsr getbyte_from_dtv
    sta endtrack+1
    lda #$00
    sta $1C03       ; port A to input
    lda #$ee 
    sta $1c0c       ; set pcr
    ; activate motor if it is needed
    lda $1c00
    and #$04
    bne starttrack
    lda $1c00
    ora #$04
    sta $1c00
    ; make delay
    ldy #$00
    ldx #$00
-   inx
    bne -
    dey
    bne -
     
starttrack:
    lda #1 ; overwritten
    jsr head_step_to
-
    jsr readtrack
    lda CURRENT_TRACK
endtrack:
    cmp #2 ; overwritten
    beq +
    jsr head_step_in
    jsr head_step_in
    inc CURRENT_TRACK
    bne - ; unconditional   
+
    jmp command_loop


;----------------------------------------------------
;
; read and transfers the track fiven in 
;          "CURRENT_TRACK"
;
;----------------------------------------------------


readtrack:
    ldx CURRENT_TRACK
    lda disk_tracks_per_sec_tab,x
    and #%01100000
    sta DISK_TEMP        
    lda $1c00
    and #%10011111
    ora DISK_TEMP
    sta $1c00
    lda disk_tracks_per_sec_tab,x
    and #%00011111
    sta SECTORS_TO_GO
    tax
    lda #$00
-
    sta floppyblockstransferred-1,x
    dex
    bne -    
    
read_next_block:
    BIT $1C00       ; SYNC signal
    bmi read_next_block ; not yet found?
    bit $1c01
    clv
	bvc *
    lda $1c01
    clv
    cmp #$52 ; header ?
    bne read_next_block
    sta floppyblock
    ldx #$01
readheaderloop:
	bvc *
    lda $1c01
    clv
    sta floppyblock,x
    inx
    cpx #$0a
    bne readheaderloop
    lda floppyblock+5
    sta DISK_TEMP
    and #$1f
    tax
    lda floppyblock+2
    and #$0f
    asl
    tax
    lda floppyblock+3
    asl
    bcc +
    inx
+   lsr
    lsr
    lsr
    tay
    
    lda disk_gcrrevtab,y
    ora disk_gcrrevtab_shifted,x
    tax
    lda floppyblockstransferred,x
    bne read_next_block
    inc floppyblockstransferred,x
        
    ldx #$00
readdataloop:
    BIT $1C00       ; SYNC signal
    bmi readdataloop ; not yet found?
    bit $1c01
    clv
	bvc *
    lda $1c01
    clv
    cmp #$55 ; header ?
    bne read_next_block
    sta floppyblock+10
    ldx #$01
readblockloop:
	bvc *
    lda $1c01
    clv
    sta floppyblock+10,x
    inx
    bne readblockloop

readblockloop2:
	bvc *
    lda $1c01
    clv
    sta floppyblock+10+$100,x
    inx
    cpx #$46
    bne readblockloop2
    
    lda $1c00
    eor #$08
    sta $1c00
            
    jsr short_break
    lda #$0a
    sta $1800
    
-
    lda $1800 ;waiting for datain = high
    bpl -
-
    lda $1800 ;waiting for datain = high
    bmi -
    
    lda #$00
    sta $1800    
    jsr short_break

    ldx #$00
copyloop1:
    lda floppyblock,x
    
    !for i,4 {
    ldy #$08
    asl
    bcc +
    ldy #$00
+
    sty $1800
    nop

    ldy #$0a
    asl
    bcc +
    ldy #$02
+
    sty $1800
    nop
}
    inx
    bne copyloop1

copyloop2:
    lda floppyblock+$100,x
    
    !for i,4 {
    ldy #$08
    asl
    bcc +
    ldy #$00
+
    sty $1800
    nop

    ldy #$0a
    asl
    bcc +
    ldy #$02
+
    sty $1800
    nop
}
    inx
    cpx #$45+$0a
    bne copyloop2
    lda #$00
    sta $1800
    dec SECTORS_TO_GO
    beq +    
    jmp read_next_block
+   rts


;----------------------------------------------------
;
; takes byte from dtv and returns it in accu
;
;----------------------------------------------------

getbyte_from_dtv:
    lda $1800
    ora #$08
    sta $1800
    ldx #$08
getbit_from_dtv
-
    lda $1800 ;waiting for datain = high
    bpl -
-
    lda $1800 ;waiting for datain = high
    bmi -
    lda $1800
    lsr
    rol DTV_BYTE
    dex
    bne getbit_from_dtv
    lda $1800
    and #$f7
    sta $1800
-   lda $1800 ;waiting for datain = high
    bpl -
-
    lda $1800 ;waiting for datain = high
    bmi -
    lda DTV_BYTE
    rts
    
;----------------------------------------------------
;
; short break used for transfer to DTV
;
;----------------------------------------------------

short_break:
    ; destroys x
    ldx #$04
-   dex
    bne -
    rts

;----------------------------------------------------
;
; error function
;
;----------------------------------------------------

error:
    lda $1c00
    and #$fb
    sta $1c00
    lda #$20
    sta DISK_TEMP
errorloop:
    lda $1c00
    eor #$08
    sta $1c00
    ldx #$80
    ldy #$00
-   dey
    bne -
    dex
    bne -
    dec DISK_TEMP
    bne errorloop
    jmp ($fffc)

;----------------------------------------------------
;
; step to track given in accu
;
;----------------------------------------------------

head_step_to:
    sta TRACK_POS_WANTED
-
    lda TRACK_POS_WANTED
    cmp CURRENT_TRACK
    beq head_step_to_end
    bcc +
    inc CURRENT_TRACK
    jsr head_step_in
    jsr head_step_in
    jmp -
+
    dec CURRENT_TRACK
    jsr head_step_out
    jsr head_step_out
    jmp -
    
;----------------------------------------------------
;
; step to higher track
;
;----------------------------------------------------

head_step_in:
    ldx $1c00
    inx
    jmp head_step

;----------------------------------------------------
;
; step to lower track
;
;----------------------------------------------------

head_step_out:
    ldx $1c00
    dex
head_step:
    txa
    and #$03
    sta DISK_TEMP
    lda $1c00
    and #$fc
    ora DISK_TEMP
    sta $1c00
    ldx #$00
    ldy #$08
-   dex
    bne -
    dey
    bne -
head_step_to_end:
    rts

;----------------------------------------------------
;
; diskchange
;
;----------------------------------------------------

diskchange:
    lda #$01
    sta DISK_TEMP
    sta DTV_BYTE
    lda $1c00
    and #$fb
    sta $1c00
    lda #$10
    sta OLD_CHANGE_STAT
    ldx #$00
    stx CHANGE_COUNTER
diskchangeblink:
    lda $1c00
    and #$10
    cmp OLD_CHANGE_STAT
    sta OLD_CHANGE_STAT
    beq nochange
    inc CHANGE_COUNTER
    lda CHANGE_COUNTER
    cmp #4
    bne nochange
    ; to synchronize with dtv
    ldx #$00
    ldy #$00
-
    dec DISK_TEMP
    inc DISK_TEMP 
    inx
    bne -
    iny
    bne -
    lda $1c00
    ora #$04
    sta $1c00
    jmp getbyte_from_dtv
nochange:
    lda $1c00
    ora #$08
    sta $1c00
    lda DISK_TEMP
    clc
    adc DTV_BYTE
    sta DISK_TEMP
    cmp #$01
    beq +
    cmp #$ff
    bne diskchangeloop
+   lda DTV_BYTE
    eor #$ff
    clc
    adc #$01
    sta DTV_BYTE
diskchangeloop:
    inx
    beq diskchangeblink
    cpx DISK_TEMP
    bne diskchangeloop
    lda $1c00
    and #$f7
    sta $1c00
    jmp diskchangeloop
    
        
    

;----------------------------------------------------
;
; tables for decoding the sector number
;
;----------------------------------------------------

disk_gcrrevtab:
    !byte -1
    !byte -1
    !byte -1
    !byte -1
    !byte -1
    !byte -1
    !byte -1
    !byte -1
    !byte -1
    !byte 8
    !byte 0
    !byte 1
    !byte -1
    !byte $c
    !byte 4
    !byte 5
    !byte -1
    !byte -1
    !byte 2
    !byte 3
    !byte -1
    !byte $f
    !byte 6
    !byte 7
    !byte -1
    !byte 9
    !byte $a
    !byte $b
    !byte -1
    !byte $d
    !byte $e
    !byte -1

disk_gcrrevtab_shifted:
    !byte -1
    !byte -1
    !byte -1
    !byte -1
    !byte -1
    !byte -1
    !byte -1
    !byte -1
    !byte -1
    !byte 8*$10
    !byte 0*$10
    !byte 1*$10
    !byte -1
    !byte $c*$10
    !byte 4*$10
    !byte 5*$10
    !byte -1
    !byte -1
    !byte 2*$10
    !byte 3*$10
    !byte -1
    !byte $f*$10
    !byte 6*$10
    !byte 7*$10
    !byte -1
    !byte 9*$10
    !byte $a*$10
    !byte $b*$10
    !byte -1
    !byte $d*$10
    !byte $e*$10
    !byte -1

;----------------------------------------------------
;
; tables for number of sectors and speed
;
;----------------------------------------------------

disk_tracks_per_sec_tab:
    ;holds also the speed value
    !byte -1
    !for i,17 {
    !byte 21+%01100000
    }
    !for i,7 {
    !byte 19+%01000000
    }
    !for i,6 {
    !byte 18+%00100000
    }
    !for i,10 {
    !byte 17+%00000000
    }


floppytop:
         
}

floppycodeend:

        
