;====================================================
;
; dtv check functions
;
; by peiselulli in 2008
;
;====================================================

;----------------------------------------------------
;
; check_ntsc
; check if dtv is in ntsc mode
;
; after call:
; a = 0 => ntsc, x and y are untouched
;
;----------------------------------------------------

check_ntsc:
              ; return value of a = 0 -> ntsc

        php
        sei                         ; disable interrupts
check_ntsc_wait:
        lda $d012
        bne check_ntsc_wait         ; wait for rasterline 0 or 256
        lda #$37
        sta $d012
        lda #$9b                    ; write testline $137 to the
        sta $d011                   ; latch-register
        lda #$01
        sta $d019                   ; clear IMR-Bit 0
check_ntsc_wait1:
        lda $d011                   ; Is rasterbeam in the area
        bpl check_ntsc_wait1        ; 0-255? if yes, wait
check_ntsc_wait2:
        lda $d011                   ; Is rasterbeam in the area
        bmi check_ntsc_wait2        ; 256 to end? if yes, wait
        lda $d019                   ; read IMR
        and #$01                    ; mask Bit 0
        sta $d019                   ; clear IMR-Bit 0
        plp                         ; enable interrupts
        rts
              
;----------------------------------------------------
;
; check_dtv
; check if dtv is in ntsc mode
;
; after call:
; a = 0 => no dtv, x and y are untouched
;
;----------------------------------------------------

check_dtv:
        +checkdtv
        bne is_dtv
check_v3_value:
        lda #$00
is_dtv: rts


;----------------------------------------------------
;
; check_v3
; check if dtv is a version 3 revision
;
; after call:
; a = 0 => version 3, x is changed, y is untouched
;
;----------------------------------------------------

check_v3:
        lda #$01
        sta $d03f
        jsr check_v3_wait_vbl
        ldx #$1f
-       lda check_blttab,x
        sta blt_a_low,x
        dex
        bpl -
        lda #$0f
        sta blt_ctrl_0
check_v3_wait_vbl:
        +waitblit
check_v3_ret:
        lda #$00
        rts
        

check_blttab:
check_tab_blt_a_low:              !byte <(check_v3_value+1) ; = $d320
check_tab_blt_a_mid:              !byte >(check_v3_value+1) ; = $d321
check_tab_blt_a_high:             !byte 0         ; = $d322
check_tab_blt_a_mod_low:          !byte 0         ; = $d323
check_tab_blt_a_mod_mid:          !byte 0         ; = $d324
check_tab_blt_a_llength_low:      !byte $ff       ; = $d325
check_tab_blt_a_llength_mid:      !byte $ff       ; = $d326
check_tab_blt_a_frac_step:        !byte 0         ; = $d327

check_tab_blt_b_low:              !byte <(check_v3_value)    ; = $d328
check_tab_blt_b_mid:              !byte >(check_v3_value+1)  ; = $d329
check_tab_blt_b_high:             !byte 0         ; = $d32a
check_tab_blt_b_mod_low:          !byte 0         ; = $d32b
check_tab_blt_b_mod_mid:          !byte 0         ; = $d32c
check_tab_blt_b_llength_low:      !byte $ff       ; = $d32d
check_tab_blt_b_llength_mid:      !byte $ff       ; = $d32e
check_tab_blt_b_frac_step:        !byte 0         ; = $d32f

check_tab_blt_d_low:              !byte <(check_v3_ret+1)     ; = $d330
check_tab_blt_d_mid:              !byte >(check_v3_ret+1)     ; = $d331
check_tab_blt_d_high:             !byte 0         ; = $d332
check_tab_blt_d_mod_low:          !byte 0         ; = $d333
check_tab_blt_d_mod_mid:          !byte 0         ; = $d334
check_tab_blt_d_llength_low:      !byte $ff       ; = $d335
check_tab_blt_d_llength_mid:      !byte $ff       ; = $d336
check_tab_blt_d_frac_step:        !byte 16        ; = $d337

check_tab_blt_length_low:         !byte 1         ; = $d338
check_tab_blt_length_mid:         !byte 0         ; = $d339
check_tab_blt_ctrl_0:             !byte 0         ; = $d33a
check_tab_blt_ctrl_1:             !byte %100      ; = $d33b
                                  !byte 0         ; = $d33c
                                  !byte 0         ; = $d33d
check_tab_blt_shift:              !byte %11000    ; = $d33e
check_tab_blt_status:             !byte 0         ; = $d33f
        
