
;------------------------------------------
; +sac : following byte is argument
; +sir : following byte is argument
; +bra : following label is argument
; +align4 : alignment to the next pc dividable by 4 with NOPs
; +align256 : alignment to the next pc dividable by 256 with NOPs
; +cyclestart : 0x4CAC= 19628 cycles per frame (pal) (used regs : a)
; +cyclestop (used regs : a)
; +cycleprint (used regs : a,x if printascii not useds y)
; +mul : A/X = A * Y (X LOW) (used regs : a,x,y)
; +initmul (used regs : a,x,y)
; +checkdtv : return flags equal if no dtv (used regs : a)
; +enable_extended : (used regs : a)
; +check_ntsc : return value of a = 0 -> ntsc (used regs : a,x,y)
; +waitblit : wait for blitter ready (used regs : a)
; +waitdma : wait for dma ready (used regs : a)
; +peek_long .adr : get value .adr, .adr+1 and .adr+2 points (uses bank $c000-$ffff)
;
;------------------------------------------

;------------------------------------------
; switches
;------------------------------------------
;cyclecount = 1

;------------------------------------------
; definitions to do in the code that includes this file
; (when the corresponding feature is used)
;------------------------------------------

;MULTAB = $a800
;printascii = $ffd2

;------------------------------------------
; names for hardware registers
;------------------------------------------

vic_lca_low = $d03a
vic_lca_mid = $d03b
vic_lca_high = $d045
vic_lca_step = $d046
vic_lca_mod_low = $d038
vic_lca_mod_mid = $d039

vic_lcb_low = $d049
vic_lcb_mid = $d04a
vic_lcb_high = $d04b
vic_lcb_step = $d04c
vic_lcb_mod_low = $d047
vic_lcb_mod_mid = $d048

blt_a_low = $d320
blt_a_mid = $d321
blt_a_high = $d322
blt_a_mod_low = $d323
blt_a_mod_mid = $d324
blt_a_llength_low = $d325
blt_a_llength_mid = $d326
blt_a_frac_step = $d327

blt_b_low = $d328
blt_b_mid = $d329
blt_b_high = $d32a
blt_b_mod_low = $d32b
blt_b_mod_mid = $d32c
blt_b_llength_low = $d32d
blt_b_llength_mid = $d32e
blt_b_frac_step = $d32f

blt_d_low = $d330
blt_d_mid = $d331
blt_d_high = $d332
blt_d_mod_low = $d333
blt_d_mod_mid = $d334
blt_d_llength_low = $d335
blt_d_llength_mid = $d336
blt_d_frac_step = $d337

blt_length_low = $d338
blt_length_mid = $d339
blt_ctrl_0 = $d33a
      ;Bit 0 : Force Start when set
      ;Bit 1 : Source A Direction (positive when set)
      ;Bit 2 : Source B Direction (positive when set)
      ;Bit 3 : Destination Direction (positive when set)
      ;Bit 4 : VIC IRQ Start when set
      ;Bit 5 : CIA IRQ Start when set ($dcxx CIA)
      ;Bit 6 : VBlank Start when set
      ;Bit 7 : Blitter IRQ enable when set
blt_ctrl_1 = $d33b
      ;Bit 0 : Disable Channel B (data from b = %00000000)
      ;Bit 1 : Write Transparent Data (write if data from a == %00000000)
      ;Bit 2 : Write non Transparent Data (write if data from a != %00000000)

blt_shift = $d33e
      ;Bit [2:0] Source A right Shift
      ;          000 Source A Data
      ;          001 Last A[0], Source A[7:1]
      ;             ...
      ;          111 Last A[6:0], Source A[7]
      ;Bit [5:3] Minterms
      ;          000 AND
      ;          001 NAND
      ;          010 NOR
      ;          011 OR
      ;          100 XOR
      ;          101 XNOR
      ;          110 a+b
      ;          111 a-b

blt_status = $d33f
      ;Write:
      ; Bit 0 : Clear Blitter IRQ
      ; Bit 1 : Source A continue
      ; Bit 2 : Source B continue
      ; Bit 3 : Destination continue
      ;Read:
      ; Bit 0 : Blitter busy
      ; Bit 1 : IRQ occurred


dma_src_low = $d300
dma_src_mid = $d301
dma_src_high = $d302
dma_dst_low = $d303
dma_dst_mid = $d304
dma_dst_high = $d305
dma_src_step_low = $d306
dma_src_step_mid = $d307
dma_dst_step_low = $d308
dma_dst_step_mid = $d309
dma_length_low = $d30a
dma_length_mid = $d30b
dma_src_mod_low = $d30c
dma_src_mod_mid = $d30d
dma_dst_mod_low = $d30e
dma_dst_mod_mid = $d30f
dma_src_llength_low = $d310
dma_src_llength_mid = $d311
dma_dst_llength_low = $d312
dma_dst_llength_mid = $d313
dma_continue = $d31d
        ;Bit 0 : Clear IRQ (set to clear IRQ)
        ;Bit 1 : SourceContinue
        ;Bit 3   DestinationContinue
dma_mod_enable = $d31e
        ;Bit 1 : Source modulo enable
        ;Bit 3   Destination modulo enable
dma_ctrl = $d31f
        ; Write :
        ;  Bit 0 : Force Start DMA when set
        ;  Bit 1 : swap source with dest
        ;  Bit 2 : Source Direction (1 = positive)
        ;  Bit 3 : Destination direction (1 = positive)
        ;  Bit 4 : VIC IRQ Start enable DMA on VIC IRQ when set
        ;  Bit 5 : Start on blitter done when set
        ;  Bit 6 : VBlank Start when set
        ;  Bit 7 : IRQ Enable
dma_status = $d31f
        ; Read :
        ;  Bit 0 : DMA busy
        ;  Bit 1 : IRQ occurred

;------------------------------------------
; dependent defintions
;------------------------------------------


;------------------------------------------
; macros
;------------------------------------------

!MACRO sac .v {
    !BYTE $32,.v
}

;------------------------------------------

!MACRO sir .v {
    !byte $42,.v
}

;------------------------------------------

!MACRO sbx .v {
    !byte $cb,.v
}

;------------------------------------------

!MACRO bra .v {
    !byte $12, (.v - 2) - *
}

;------------------------------------------

!MACRO align4 {
    !if (* & 03) <> 0 {
    nop
    }
    !if (* & 03) <> 0 {
    nop
    }
    !if (* & 03) <> 0 {
    nop
    }
}

;------------------------------------------

!MACRO check_align4 {
    !if (* & 03) <> 0 {
    !error -47  ;not aligned
    }
}

;------------------------------------------

!MACRO align256 {
    !for i,256 {
      !if (* & $ff) <> 0 {
      nop
      }
    }
}

;------------------------------------------

!MACRO cyclestart {
    !if cyclecount <> 0 {
    lda #$00
    sta $dd03
    sta $dd0f
    lda #$ff
    sta $dd04
    sta $dd05
    sta $dd06
    sta $dd07
    lda #$51
    sta $dd0f
    lda #$11
    sei

    +align4
    sta $dd0e
    php
    +align4
    nop
    nop
    }
}

;------------------------------------------


!MACRO cyclestop {
    !if cyclecount <> 0 {
    lda #$00
    +align4
    sta $dd0e
    sta $dd0f
    plp
    }
}

;------------------------------------------

!MACRO cycleprint {
    !if cyclecount <> 0 {
    lda $dd04
    clc
    adc #$05
    pha
    lda $dd05
    adc #$00
    pha
    lda $dd06
    adc #$00
    pha
    lda $dd07
    adc #$00
    eor #$ff
    jsr .printhex
    pla
    eor #$ff
    jsr .printhex
    pla
    eor #$ff
    jsr .printhex
    pla
    tax
    lda #>.printhexend-1
    pha
    lda #<.printhexend-1
    pha
    txa
    eor #$ff
.printhex:
    pha
    lsr
    lsr
    lsr
    lsr
    jsr .printnibble
    pla
    and #$0f
        
.printnibble:
    cmp #$0a
    bcc .printnibble2
    adc #6
.printnibble2:
    adc #$30
    jmp printascii
.printhexend:
    }
}



;------------------------------------------

!MACRO mul {
.MULTAB1  = MULTAB
.MULTAB2  = .MULTAB1+$0200
.MULTAB3  = .MULTAB2+$0200
.MULTAB4  = .MULTAB3+$0200
         ;A/X = A * Y (X LOW)
         STA .MULZP1+1
         STA .MULZP2+1
         EOR #$FF
         STA .MULZP3+1
         STA .MULZP4+1
         SEC
.MULZP1
         LDA .MULTAB1,Y
.MULZP3
         SBC .MULTAB3,Y
         TAX
.MULZP2
         LDA .MULTAB2,Y
.MULZP4
         SBC .MULTAB4,Y
}

;------------------------------------------

!MACRO initmul {
.MULTAB1  = MULTAB
.MULTAB2  = .MULTAB1+$0200
.MULTAB3  = .MULTAB2+$0200
.MULTAB4  = .MULTAB3+$0200

         LDX #0
         STX .MULTAB3+$FE
         STX .MULTAB4+$FE
         LDY #$FF
.INITMULL1
         TXA
         LSR
         CLC
         ADC .MULTAB3+$FE,X
         STA .MULTAB1,X
         STA .MULTAB3+$FF,X
         STA .MULTAB3,Y
         LDA #0
         ADC .MULTAB4+$FE,X
         STA .MULTAB2,X
         STA .MULTAB4+$FF,X
         STA .MULTAB4,Y
         DEY
         INX
         BNE .INITMULL1
.INITMULL2
         TXA
         SEC
         ROR
         CLC
         ADC .MULTAB1+$FF,X
         STA .MULTAB1+$0100,X
         LDA #0
         ADC .MULTAB2+$FF,X
         STA .MULTAB2+$0100,X
         INX
         BNE .INITMULL2
}

;------------------------------------------

 
!MACRO checkdtv {
; return equal if no dtv
        lda #$01
        sta $d03f
        sta $d000
        cmp $d040
}

;------------------------------------------

!MACRO enable_extended {
         ;ENABLE EXTENDED
         LDA #$01
         STA $D03F
         ;ENABLE TURBO
         +sac $99
         LDA #$03
         +sac $00
         ;DISABLE BADLINE
         lda #$20
         sta $d03c
}
        

;------------------------------------------

!MACRO check_ntsc {

nmivec        = $0318                     ; NMI-vector
              ; return value of a = 0 -> ntsc
              php
              sei                         ; disable interrupts
              ldx nmivec
              ldy nmivec+1                ; remember old NMI-vector
              lda #<(rti_vec)
              sta nmivec
              lda #>(rti_vec)                ; let NMI-vector point to
              sta nmivec+1                ; a RTI
wait:
              lda $d012
              bne wait                    ; wait for rasterline 0 or 256
              lda #$37
              sta $d012
              lda #$9b                    ; write testline $137 to the
              sta $d011                   ; latch-register
              lda #$01
              sta $d019                   ; clear IMR-Bit 0
wait1:
              lda $d011                   ; Is rasterbeam in the area
              bpl wait1                   ; 0-255? if yes, wait
wait2:
              lda $d011                   ; Is rasterbeam in the area
              bmi wait2                   ; 256 to end? if yes, wait
              lda $d019                   ; read IMR
              and #$01                    ; mask Bit 0
              sta $d019                   ; clear IMR-Bit 0
              stx nmivec
              sty nmivec+1                ; restore old NMI-vector
              plp                         ; enable interrupts
              !byte $24                   ; bit zp

rti_vec:          rti                         ; go immediately back after
                                          ; a NMI
}

;------------------------------------------
!MACRO waitblit {
-       lda blt_status
        lsr
        bcs -
}

;------------------------------------------

!MACRO waitdma {
-       lda dma_status
        lsr
        bcs -
}

;------------------------------------------

!MACRO lea .ptr, .val {
    lda #<.val
    sta .ptr
    !if ((<.val) <> (>.val)) {
    lda #>.val
    }
    sta .ptr+1
    !if ((>.val) <> (^.val)) {
    lda #^.val
    }
    sta .ptr+2
}

;------------------------------------------

!MACRO move_long .ptr1, .ptr2 {
    lda .ptr1
    sta .ptr2
    lda .ptr1+1
    sta .ptr2+1
    lda .ptr1+2
    sta .ptr2+2
}

;------------------------------------------

!MACRO lda_long .adr {
    php
    sei
    +sir $10
    ldx .adr
    +sac $ff
    pha
    lda .adr+2
    +sac $aa
    pha
    lda .adr+1    
 
    asl
    +sac $ff    
    rol
    +sac $aa
    asl
    +sac $ff
    rol
    +sac $aa
    sec
    ror
    sec
    ror
    +sac $00
    lda 00,x
    +sir $12
    +sac $aa
    pla
    +sac $ff
    pla
    +sac $00
    plp
}

;------------------------------------------

!MACRO add_long_imm .adr, .val {
    !if (.val < 2) {
    inc .adr
    bne +
    inc .adr+1
    bne +
    inc .adr+2
    } else {    
    !if (.val < 0x100) {
    lda .adr
    clc
    adc #<.val
    sta .adr
    bcc +
    inc .adr+1
    bne +
    inc .adr+2
    } else {
    lda .adr
    clc
    adc #<.val
    sta .adr
    lda .adr+1
    adc #>.val
    sta .adr+1
    !if (.val < 0x10000) {
    bcc +
    inc .adr+2
    } else {
    lda .adr+2
    adc #^.val
    sta .adr+2
    }
    }
    }
+
}

;------------------------------------------

!MACRO add_long .ptr1, .ptr2 {
    lda .ptr1
    clc
    adc .ptr2
    sta .ptr1
    lda .ptr1+1
    adc .ptr2+1
    sta .ptr1+1
    lda .ptr1+2
    adc .ptr2+2
    sta .ptr1+2
}

;------------------------------------------

!MACRO add_long_a .ptr {
    clc
    adc .ptr
    sta .ptr
    bcc +
    inc .ptr+1
    bne +
    inc .ptr+2
+
}

;------------------------------------------

!MACRO asl_long .ptr {
    asl .ptr
    rol .ptr+1
    rol .ptr+2
}

;------------------------------------------

!MACRO neg_long .ptr {
    lda .ptr
    eor #$ff
    clc
    adc #$01
    sta .ptr
    lda .ptr+1
    eor #$ff
    adc #$00
    sta .ptr+1
    lda .ptr+2
    eor #$ff
    adc #$00
    sta .ptr+2
}

;------------------------------------------

!MACRO store_y_postinc .ptr1, .ptr2 {
    lda .ptr1
    sta (.ptr2),y
    iny
    bne .l1
    inc .ptr2+1
.l1
    lda .ptr1+1
    sta (.ptr2),y
    iny
    bne .l2
    inc .ptr2+1
.l2
    lda .ptr1+2
    sta (.ptr2),y
    iny
    bne .l3
    inc .ptr2+1
.l3   
}

;------------------------------------------

!MACRO convert_to_segment_ptr .ptr, .segment {
    lda .ptr+1
    asl
    rol .ptr+2
    asl
    rol .ptr+2
    !if ((.segment & 1) = 1) {
    sec
    } else {
    clc
    }
    ror
    !if ((.segment & 2) = 2) {
    sec
    } else {
    clc
    }
    ror
    sta .ptr+1
}


;------------------------------------------

!MACRO convert_from_segment_ptr .ptr {
    lda .ptr+1
    asl
    asl
    lsr .ptr+2
    ror
    lsr .ptr+2
    ror
    sta .ptr+1
}

;------------------------------------------

!MACRO bne .adr {
.val = (.adr - 2) - *
    !if (.val < 128) {
        beq +
        jmp .adr
    } else {
    !if (.val > 127) {
        beq +
        jmp .adr
    } else {
        bne .adr
    }
    }
+
}

;------------------------------------------

!MACRO beq .adr {
.val = (.adr - 2) - *
    !if (.val < 128) {
        bne +
        jmp .adr
    } else {
    !if (.val > 127) {
        bne +
        jmp .adr
    } else {
        beq .adr
    }
    }
+
}

;------------------------------------------

!MACRO bcc .adr {
.val = (.adr - 2) - *
    !if (.val < 128) {
        bcs +
        jmp .adr
    } else {
    !if (.val > 127) {
        bcs +
        jmp .adr
    } else {
        bcc .adr
    }
    }
+
}

;------------------------------------------

!MACRO bcs .adr {
.val = (.adr - 2) - *
    !if (.val < 128) {
        bcc +
        jmp .adr
    } else {
    !if (.val > 127) {
        bcc +
        jmp .adr
    } else {
        bcs .adr
    }
    }
+
}

;------------------------------------------

!MACRO bpl .adr {
.val = (.adr - 2) - *
    !if (.val < 128) {
        bmi +
        jmp .adr
    } else {
    !if (.val > 127) {
        bmi +
        jmp .adr
    } else {
        bpl .adr
    }
    }
+
}

;------------------------------------------

!MACRO bmi .adr {
.val = (.adr - 2) - *
    !if (.val < 128) {
        bpl +
        jmp .adr
    } else {
    !if (.val > 127) {
        bpl +
        jmp .adr
    } else {
        bmi .adr
    }
    }
+
}

;------------------------------------------

!MACRO sty_abs .adr {
    !byte $8c, <.adr, >.adr
}

;------------------------------------------

!MACRO ldy_abs .adr {
    !byte $ac, <.adr, >.adr
}

;------------------------------------------

!MACRO sta_abs .adr {
    !byte $8d, <.adr, >.adr
}

;------------------------------------------

!MACRO lda_abs .adr {
    !byte $ad, <.adr, >.adr
}

;------------------------------------------

!MACRO ora_abs .adr {
    !byte $0d, <.adr, >.adr
}

;------------------------------------------

!MACRO rol_abs .adr {
    !byte $2e, <.adr, >.adr
}

;------------------------------------------

!MACRO dec_abs .adr {
    !byte $ce, <.adr, >.adr
}
