;====================================================
;
; dtv demo kernal include file
;
; by peiselulli in 2008
;
;====================================================


;----------------------------------------------------
;
; kernal interface variables
;
;----------------------------------------------------

kernal_base = $c000

; the loader buffer need the memory at $008000-$00bfff !

; for files

load_addr_low = $56
load_addr_mid = $57
load_addr_high = $58
load_addr_ignore = $59

; for direct access

disk_start_track = $49
disk_end_track = $4a

dest_addr_low_byte = $4b
dest_addr_mid_byte = $4c
dest_addr_high_byte = $4d

; for depacking & copying

packed_src_low = $42
packed_src_mid = packed_src_low+1
packed_src_high = packed_src_mid+1

packed_dst_low = $45
packed_dst_mid = packed_dst_low+1
packed_dst_high = packed_dst_mid+1

memcpy_src_low = $42
memcpy_src_mid = memcpy_src_low+1
memcpy_src_high = memcpy_src_mid+1

memcpy_dst_low = $45
memcpy_dst_mid = memcpy_dst_low+1
memcpy_dst_high = memcpy_dst_mid+1


memcpy_len_low = $48
memcpy_len_mid = memcpy_len_low+1
memcpy_len_high = memcpy_len_mid+1


;----------------------------------------------------
; file getbyte
; in 'a' byte returned
; carry set if no character left
; sprite and colorram DMA access must be DISABLED !
; file must be opened with "file_open" before
;
;----------------------------------------------------
file_getbyte = $c000

;----------------------------------------------------
; file_open
; in a/x name (a low)
; if load_addr_ignore is non-zero, don't skip start address
; carry set -> failed
; sprite and colorram DMA access must be DISABLED !
; "fastload_initialize" must be called before
;
;----------------------------------------------------
file_open = kernal_base+$3

;----------------------------------------------------
; file_load
; in a/x name (a low) (with wildcard "*" and "?" possible),
;    'name' '0'-terminated
; in load_addr_low, load_addr_mid, load_addr_high
;     start addr
;  if all are zero, take the original one (load_addr_ignore must be 0)
; if load_addr_ignore is non-zero, don't skip start address
; sprite and colorram DMA access must be DISABLED !
; "fastload_initialize" must be called before
;
;----------------------------------------------------
file_load = kernal_base+$6

;----------------------------------------------------
;
; check_disk_name
; check if diskname comparison is good (with wildcard "*" and "?" possible) 
; in a/x pointer (a low)
; return: 
; a = 0, zero set -> name ok
; a != 0, zero clear -> name not ok
; sprite and colorram DMA access must be DISABLED !
; "fastload_initialize" must be called before
;
;----------------------------------------------------
check_disk_name = kernal_base+$9

;----------------------------------------------------
;
; copy code to floppy and execute it
; no arguments
;
;----------------------------------------------------        
fastload_initialize = kernal_base+$c

;----------------------------------------------------
;
; disk_change
; wait for disk change
; no arguments
; "fastload_initialize" must be called before
;
;----------------------------------------------------
disk_change = kernal_base+$f

;----------------------------------------------------
;
; led_off
; led from floppy drive off
; no arguments
; "fastload_initialize" must be called before
;
;----------------------------------------------------
led_off = kernal_base+$12

;----------------------------------------------------
;
; led_on
; led from floppy drive on
; no arguments
; "fastload_initialize" must be called before
;
;----------------------------------------------------
led_on = kernal_base+$15

;----------------------------------------------------
;
; motor_off
; switch of motor immidiate
; no arguments
; "fastload_initialize" must be called before
;
;----------------------------------------------------
motor_off = kernal_base+$18

;----------------------------------------------------
;
; floppy_reset
; resets the disk drive, no other function accessible later
; no arguments
; "fastload_initialize" must be called before
;
;----------------------------------------------------
floppy_reset = kernal_base+$1b

;----------------------------------------------------
;
; read_tracks
; in dest_addr_low_byte/dest_addr_mid_byte/dest_addr_high_byte 
;   address to load
; in disk_start_track and disk_end_track start and end (inclusive)
; sprite and colorram DMA access must be DISABLED !
; "fastload_initialize" must be called before
;
;----------------------------------------------------
read_tracks = kernal_base+$1e

;----------------------------------------------------
;
; depack
; depacker for exomize
; in packed_src_low/packed_src_mid/packed_src_high 
;   address of the packed source
; in packed_dst_low/packed_dst_mid/packed_dst_high
;   address to depack to
; sprite and colorram DMA access must be DISABLED !
;
; literals are not supported (use -c for packing)
; CAUTION : this depacker enabled interrupt while depacking
;           so disable all IRQ sources or set up a proper
;           irq function !
;
;----------------------------------------------------
depack = kernal_base+$21

;----------------------------------------------------
;
; memcpy
; fast memcpy with blitter
; in memcpy_src_low/memcpy_src_mid/memcpy_src_high 
;   address of source data
; in memcpy_dst_low/memcpy_dst_mid/memcpy_dst_high
;   address of destination data
; in memcpy_len_low/memcpy_len_mid/memcpy_len_high
;   length of data
;
; a and x register is changed, y is untouched
;
;----------------------------------------------------
memcpy = kernal_base+$24


;----------------------------------------------------
;
; memset
; fast memset with blitter
; in a value to set
; in memcpy_dst_low/memcpy_dst_mid/memcpy_dst_high
;   address of destination data
; in memcpy_len_low/memcpy_len_mid/memcpy_len_high
;   length of data
;
; a and x register is changed, y is untouched
;
;----------------------------------------------------
memset = kernal_base+$27

;----------------------------------------------------
;
; check_ntsc
; check if dtv is in ntsc mode
;
; after call:
; a = 0 => ntsc, x and y are untouched
;
;----------------------------------------------------
check_ntsc = kernal_base+$2a

;----------------------------------------------------
;
; check_dtv
; check if dtv is in ntsc mode
;
; after call:
; a = 0 => no dtv, x and y are untouched
;
;----------------------------------------------------
check_dtv = kernal_base+$2d

;----------------------------------------------------
;
; check_v3
; check if dtv is a version 3 revision
;
; after call:
; a = 0 => version 3, x is changed, y is untouched
;
;----------------------------------------------------
check_v3 = kernal_base+$30


