;====================================================
;
; dtv demo kernal top memcpy/memset with blitter
;
; by peiselulli in 2008
;
;====================================================



memcpy_src_low = $42
memcpy_src_mid = memcpy_src_low+1
memcpy_src_high = memcpy_src_mid+1

memcpy_dst_low = $45
memcpy_dst_mid = memcpy_dst_low+1
memcpy_dst_high = memcpy_dst_mid+1


memcpy_len_low = $48
memcpy_len_mid = memcpy_len_low+1
memcpy_len_high = memcpy_len_mid+1

!macro copy_blttab {
    ldx #$1f
-   lda blttab,x
    sta $d320,x
    dex
    bpl -
}

;----------------------------------------------------
;
; memcpy
; fast memcpy with blitter
; in memcpy_src_low/memcpy_src_mid/memcpy_src_high 
;   address of source data
; in memcpy_dst_low/memcpy_dst_mid/memcpy_dst_high
;   address of destination data
; in memcpy_len_low/memcpy_len_mid/memcpy_len_high
;   length of data
;
; a and x register is changed, y is untouched
;
;----------------------------------------------------

memcpy:
    lda #16
    sta tab_blt_a_frac_step ; for copy src a increment
    +move_long memcpy_src_low, tab_blt_a_low
    +move_long memcpy_dst_low, tab_blt_d_low
memcpy_loop:
    lda memcpy_len_high
    bne nolast
    lda memcpy_len_mid
    bmi nolast
memcopy_last:
    lda memcpy_len_low
    sta tab_blt_length_low
    lda memcpy_len_mid
    sta tab_blt_length_mid
    ora memcpy_len_low
    beq no_copy ; length is zero
    +waitblit
    +copy_blttab
    LDA #%1011
    STA blt_ctrl_0
no_copy:
    +waitblit
    ldx #$1f
    lda #$00
-   sta $d320,x
    dex
    bpl -
    rts

nolast:
    lda #$00
    sta tab_blt_length_low
    lda #$80
    sta tab_blt_length_mid 
    +waitblit
    +copy_blttab 
    LDA #%1011
    STA blt_ctrl_0
    lda tab_blt_a_mid
    clc
    adc #$80
    sta tab_blt_a_mid
    bcc +
    inc tab_blt_a_high
+
    lda tab_blt_d_mid
    clc
    adc #$80
    sta tab_blt_d_mid
    bcc +
    inc tab_blt_d_high
+
    lda memcpy_len_mid
    sec
    sbc #$80
    sta memcpy_len_mid
    bcs +
    dec memcpy_len_high
+
    jmp memcpy_loop

blt_a_byte:                 !byte 0

;----------------------------------------------------
;
; memset
; fast memset with blitter
; in a value to set
; in memcpy_dst_low/memcpy_dst_mid/memcpy_dst_high
;   address of destination data
; in memcpy_len_low/memcpy_len_mid/memcpy_len_high
;   length of data
;
; a and x register is changed, y is untouched
;
;----------------------------------------------------
    
memset:
    sta blt_a_byte
    lda #$00
    sta tab_blt_a_frac_step ; for memset, same byte
    +lea tab_blt_a_low, blt_a_byte   
    +move_long memcpy_dst_low, tab_blt_d_low
memset_loop:
    lda memcpy_len_high
    bne nolast_memset
    lda memcpy_len_mid
    bmi nolast_memset
    jmp memcopy_last

nolast_memset:
    lda #$00
    sta tab_blt_length_low
    lda #$80
    sta tab_blt_length_mid 
    +waitblit
    +copy_blttab 
    LDA #%1011
    STA blt_ctrl_0
    lda tab_blt_d_mid
    clc
    adc #$80
    sta tab_blt_d_mid
    bcc +
    inc tab_blt_d_high
+
    lda memcpy_len_mid
    sec
    sbc #$80
    sta memcpy_len_mid
    bcs +
    dec memcpy_len_high
+
    +bra memset_loop
    

blttab:
tab_blt_a_low:              !byte 0         ; = $d320
tab_blt_a_mid:              !byte 0         ; = $d321
tab_blt_a_high:             !byte 0         ; = $d322
tab_blt_a_mod_low:          !byte 0         ; = $d323
tab_blt_a_mod_mid:          !byte 0         ; = $d324
tab_blt_a_llength_low:      !byte $ff       ; = $d325
tab_blt_a_llength_mid:      !byte $ff       ; = $d326
tab_blt_a_frac_step:        !byte 16        ; = $d327

tab_blt_b_low:              !byte 0         ; = $d328
tab_blt_b_mid:              !byte 0         ; = $d329
tab_blt_b_high:             !byte 0         ; = $d32a
tab_blt_b_mod_low:          !byte 0         ; = $d32b
tab_blt_b_mod_mid:          !byte 0         ; = $d32c
tab_blt_b_llength_low:      !byte 0         ; = $d32d
tab_blt_b_llength_mid:      !byte 0         ; = $d32e
tab_blt_b_frac_step:        !byte 0         ; = $d32f

tab_blt_d_low:              !byte 0         ; = $d330
tab_blt_d_mid:              !byte 0         ; = $d331
tab_blt_d_high:             !byte 0         ; = $d332
tab_blt_d_mod_low:          !byte 0         ; = $d333
tab_blt_d_mod_mid:          !byte 0         ; = $d334
tab_blt_d_llength_low:      !byte $ff       ; = $d335
tab_blt_d_llength_mid:      !byte $ff       ; = $d336
tab_blt_d_frac_step:        !byte 16        ; = $d337

tab_blt_length_low:         !byte 0         ; = $d338
tab_blt_length_mid:         !byte 0         ; = $d339
tab_blt_ctrl_0:             !byte 0         ; = $d33a
tab_blt_ctrl_1:             !byte %001      ; = $d33b
                            !byte 0         ; = $d33c
                            !byte 0         ; = $d33d
tab_blt_shift:              !byte %11000    ; = $d33e
tab_blt_status:             !byte 0         ; = $d33f

       
    
